/**
 * @debt circular "Remove & prevent imports from entity to schema"
 */
import type { UpdateValueInput } from '../../entity/actions/update/types.js';
import type { Paths, SchemaAction, ValidValue } from '../../schema/index.js';
import type { If, NarrowObject, Overwrite, ValueOrGetter } from '../../types/index.js';
import type { Always, AtLeastOnce, Never, Schema, SchemaProps, SchemaRequiredProp, Validator } from '../types/index.js';
import type { Light } from '../utils/light.js';
import { SetSchema } from './schema.js';
import type { SetElementSchema } from './types.js';
type SetSchemer = <ELEMENTS extends SetElementSchema, PROPS extends SchemaProps = {}>(elements: ELEMENTS, props?: NarrowObject<PROPS>) => SetSchema_<Light<ELEMENTS>, PROPS>;
/**
 * Define a new set attribute
 * Note that set elements have constraints. They must be:
 * - Required (required: AtLeastOnce)
 * - Displayed (hidden: false)
 * - Not renamed (savedAs: undefined)
 * - Not defaulted (defaults: undefined)
 *
 * @param elements Attribute (With constraints)
 * @param props _(optional)_ List Props
 */
export declare const set: SetSchemer;
/**
 * Set attribute interface
 */
export declare class SetSchema_<ELEMENTS extends SetElementSchema = SetElementSchema, PROPS extends SchemaProps = SchemaProps> extends SetSchema<ELEMENTS, PROPS> {
    /**
     * Tag attribute as required. Possible values are:
     * - `'atLeastOnce'` _(default)_: Required in PUTs, optional in UPDATEs
     * - `'never'`: Optional in PUTs and UPDATEs
     * - `'always'`: Required in PUTs and UPDATEs
     *
     * @param nextRequired SchemaRequiredProp
     */
    required<NEXT_IS_REQUIRED extends SchemaRequiredProp = AtLeastOnce>(nextRequired?: NEXT_IS_REQUIRED): SetSchema_<ELEMENTS, Overwrite<PROPS, {
        required: NEXT_IS_REQUIRED;
    }>>;
    /**
     * Shorthand for `required('never')`
     */
    optional(): SetSchema_<ELEMENTS, Overwrite<PROPS, {
        required: Never;
    }>>;
    /**
     * Hide attribute after fetch commands and formatting
     */
    hidden<NEXT_HIDDEN extends boolean = true>(nextHidden?: NEXT_HIDDEN): SetSchema_<ELEMENTS, Overwrite<PROPS, {
        hidden: NEXT_HIDDEN;
    }>>;
    /**
     * Tag attribute as a primary key attribute or linked to a primary attribute
     */
    key<NEXT_KEY extends boolean = true>(nextKey?: NEXT_KEY): SetSchema_<ELEMENTS, Overwrite<PROPS, {
        key: NEXT_KEY;
        required: Always;
    }>>;
    /**
     * Rename attribute before save commands
     */
    savedAs<NEXT_SAVED_AS extends string | undefined>(nextSavedAs: NEXT_SAVED_AS): SetSchema_<ELEMENTS, Overwrite<PROPS, {
        savedAs: NEXT_SAVED_AS;
    }>>;
    /**
     * Provide a default value for attribute in Primary Key computing
     *
     * @param nextKeyDefault `keyAttributeInput | (() => keyAttributeInput)`
     */
    keyDefault(nextKeyDefault: ValueOrGetter<ValidValue<this, {
        mode: 'key';
    }>>): SetSchema_<ELEMENTS, Overwrite<PROPS, {
        keyDefault: unknown;
    }>>;
    /**
     * Provide a default value for attribute in PUT commands
     *
     * @param nextPutDefault `putAttributeInput | (() => putAttributeInput)`
     */
    putDefault(nextPutDefault: ValueOrGetter<ValidValue<this>>): SetSchema_<ELEMENTS, Overwrite<PROPS, {
        putDefault: unknown;
    }>>;
    /**
     * Provide a default value for attribute in UPDATE commands
     *
     * @param nextUpdateDefault `updateAttributeInput | (() => updateAttributeInput)`
     */
    updateDefault(nextUpdateDefault: ValueOrGetter<UpdateValueInput<this, {
        filled: true;
    }>>): SetSchema_<ELEMENTS, Overwrite<PROPS, {
        updateDefault: unknown;
    }>>;
    /**
     * Provide a default value for attribute in PUT commands OR Primary Key computing if attribute is tagged as key
     *
     * @param nextDefault `key/putAttributeInput | (() => key/putAttributeInput)`
     */
    default(nextDefault: ValueOrGetter<If<PROPS['key'], ValidValue<this, {
        mode: 'key';
    }>, ValidValue<this>>>): If<PROPS['key'], SetSchema_<ELEMENTS, Overwrite<PROPS, {
        keyDefault: unknown;
    }>>, SetSchema_<ELEMENTS, Overwrite<PROPS, {
        putDefault: unknown;
    }>>>;
    /**
     * Provide a **linked** default value for attribute in Primary Key computing
     *
     * @param nextKeyLink `keyAttributeInput | ((keyInput) => keyAttributeInput)`
     */
    keyLink<SCHEMA extends Schema>(nextKeyLink: (keyInput: ValidValue<SCHEMA, {
        mode: 'key';
        defined: true;
    }>) => ValidValue<this, {
        mode: 'key';
    }>): SetSchema_<ELEMENTS, Overwrite<PROPS, {
        keyLink: unknown;
    }>>;
    /**
     * Provide a **linked** default value for attribute in PUT commands
     *
     * @param nextPutLink `putAttributeInput | ((putItemInput) => putAttributeInput)`
     */
    putLink<SCHEMA extends Schema>(nextPutLink: (putItemInput: ValidValue<SCHEMA, {
        defined: true;
    }>) => ValidValue<this>): SetSchema_<ELEMENTS, Overwrite<PROPS, {
        putLink: unknown;
    }>>;
    /**
     * Provide a **linked** default value for attribute in UPDATE commands
     *
     * @param nextUpdateLink `unknown | ((updateItemInput) => updateAttributeInput)`
     */
    updateLink<SCHEMA extends Schema>(nextUpdateLink: (updateItemInput: UpdateValueInput<SCHEMA, {
        defined: true;
        extended: false;
    }, Paths<SCHEMA>>) => UpdateValueInput<this, {
        filled: true;
    }>): SetSchema_<ELEMENTS, Overwrite<PROPS, {
        updateLink: unknown;
    }>>;
    /**
     * Provide a **linked** default value for attribute in PUT commands OR Primary Key computing if attribute is tagged as key
     *
     * @param nextLink `key/putAttributeInput | (() => key/putAttributeInput)`
     */
    link<SCHEMA extends Schema>(nextLink: (keyOrPutItemInput: If<PROPS['key'], ValidValue<SCHEMA, {
        mode: 'key';
        defined: true;
    }>, ValidValue<SCHEMA, {
        defined: true;
    }>>) => If<PROPS['key'], ValidValue<this, {
        mode: 'key';
    }>, ValidValue<this>>): If<PROPS['key'], SetSchema_<ELEMENTS, Overwrite<PROPS, {
        keyLink: unknown;
    }>>, SetSchema_<ELEMENTS, Overwrite<PROPS, {
        putLink: unknown;
    }>>>;
    /**
     * Provide a custom validator for attribute in Primary Key computing
     *
     * @param nextKeyValidator `(keyAttributeInput) => boolean | string`
     */
    keyValidate(nextKeyValidator: Validator<ValidValue<this, {
        mode: 'key';
        defined: true;
    }>, this>): SetSchema_<ELEMENTS, Overwrite<PROPS, {
        keyValidator: Validator;
    }>>;
    /**
     * Provide a custom validator for attribute in PUT commands
     *
     * @param nextPutValidator `(putAttributeInput) => boolean | string`
     */
    putValidate(nextPutValidator: Validator<ValidValue<this, {
        defined: true;
    }>, this>): SetSchema_<ELEMENTS, Overwrite<PROPS, {
        putValidator: Validator;
    }>>;
    /**
     * Provide a custom validator for attribute in UPDATE commands
     *
     * @param nextUpdateValidator `(updateAttributeInput) => boolean | string`
     */
    updateValidate(nextUpdateValidator: Validator<UpdateValueInput<this, {
        filled: true;
    }>, this>): SetSchema_<ELEMENTS, Overwrite<PROPS, {
        updateValidator: Validator;
    }>>;
    /**
     * Provide a custom validator for attribute in PUT commands OR Primary Key computing if attribute is tagged as key
     *
     * @param nextValidator `(key/putAttributeInput) => boolean | string`
     */
    validate(nextValidator: Validator<If<PROPS['key'], ValidValue<this, {
        mode: 'key';
        defined: true;
    }>, ValidValue<this, {
        defined: true;
    }>>, this>): If<PROPS['key'], SetSchema_<ELEMENTS, Overwrite<PROPS, {
        keyValidator: Validator;
    }>>, SetSchema_<ELEMENTS, Overwrite<PROPS, {
        putValidator: Validator;
    }>>>;
    clone<NEXT_PROPS extends SchemaProps = {}>(nextProps?: NarrowObject<NEXT_PROPS>): SetSchema_<ELEMENTS, Overwrite<PROPS, NEXT_PROPS>>;
    build<ACTION extends SchemaAction<this> = SchemaAction<this>>(Action: new (schema: this) => ACTION): ACTION;
}
export {};
