import { ifThenElse } from '../../utils/ifThenElse.js';
import { overwrite } from '../../utils/overwrite.js';
import { lightTuple } from '../utils/light.js';
import { AnyOfSchema } from './schema.js';
/**
 * Define a new anyOf attribute
 * @param elements Attribute[]
 */
export const anyOf = (...elements) => new AnyOfSchema_(lightTuple(...elements), {});
/**
 * AnyOf attribute interface
 */
export class AnyOfSchema_ extends AnyOfSchema {
    /**
     * Tag attribute as required. Possible values are:
     * - `'atLeastOnce'` _(default)_: Required in PUTs, optional in UPDATEs
     * - `'never'`: Optional in PUTs and UPDATEs
     * - `'always'`: Required in PUTs and UPDATEs
     *
     * @param nextRequired SchemaRequiredProp
     */
    required(nextRequired = 'atLeastOnce') {
        return new AnyOfSchema_(this.elements, overwrite(this.props, { required: nextRequired }));
    }
    /**
     * Shorthand for `required('never')`
     */
    optional() {
        return this.required('never');
    }
    /**
     * Hide attribute after fetch commands and formatting
     */
    hidden(nextHidden = true) {
        return new AnyOfSchema_(this.elements, overwrite(this.props, { hidden: nextHidden }));
    }
    /**
     * Tag attribute as a primary key attribute or linked to a primary attribute
     */
    key(nextKey = true) {
        return new AnyOfSchema_(this.elements, overwrite(this.props, { key: nextKey, required: 'always' }));
    }
    /**
     * Rename attribute before save commands
     */
    savedAs(nextSavedAs) {
        return new AnyOfSchema_(this.elements, overwrite(this.props, { savedAs: nextSavedAs }));
    }
    /**
     * Discriminates the union with a shared string attribute with enum
     */
    discriminate(nextDiscriminator) {
        return new AnyOfSchema_(this.elements, overwrite(this.props, { discriminator: nextDiscriminator }));
    }
    /**
     * Provide a default value for attribute in Primary Key computing
     *
     * @param nextKeyDefault `keyAttributeInput | (() => keyAttributeInput)`
     */
    keyDefault(nextKeyDefault) {
        return new AnyOfSchema_(this.elements, overwrite(this.props, { keyDefault: nextKeyDefault }));
    }
    /**
     * Provide a default value for attribute in PUT commands
     *
     * @param nextPutDefault `putAttributeInput | (() => putAttributeInput)`
     */
    putDefault(nextPutDefault) {
        return new AnyOfSchema_(this.elements, overwrite(this.props, { putDefault: nextPutDefault }));
    }
    /**
     * Provide a default value for attribute in UPDATE commands
     *
     * @param nextUpdateDefault `updateAttributeInput | (() => updateAttributeInput)`
     */
    updateDefault(nextUpdateDefault) {
        return new AnyOfSchema_(this.elements, overwrite(this.props, { updateDefault: nextUpdateDefault }));
    }
    /**
     * Provide a default value for attribute in PUT commands OR Primary Key computing if attribute is tagged as key
     *
     * @param nextDefault `key/putAttributeInput | (() => key/putAttributeInput)`
     */
    default(nextDefault) {
        return ifThenElse(this.props.key, new AnyOfSchema_(this.elements, overwrite(this.props, { keyDefault: nextDefault })), new AnyOfSchema_(this.elements, overwrite(this.props, { putDefault: nextDefault })));
    }
    /**
     * Provide a **linked** default value for attribute in Primary Key computing
     *
     * @param nextKeyLink `keyAttributeInput | ((keyInput) => keyAttributeInput)`
     */
    keyLink(nextKeyLink) {
        return new AnyOfSchema_(this.elements, overwrite(this.props, { keyLink: nextKeyLink }));
    }
    /**
     * Provide a **linked** default value for attribute in PUT commands
     *
     * @param nextPutLink `putAttributeInput | ((putItemInput) => putAttributeInput)`
     */
    putLink(nextPutLink) {
        return new AnyOfSchema_(this.elements, overwrite(this.props, { putLink: nextPutLink }));
    }
    /**
     * Provide a **linked** default value for attribute in UPDATE commands
     *
     * @param nextUpdateLink `unknown | ((updateItemInput) => updateAttributeInput)`
     */
    updateLink(nextUpdateLink) {
        return new AnyOfSchema_(this.elements, overwrite(this.props, { updateLink: nextUpdateLink }));
    }
    /**
     * Provide a **linked** default value for attribute in PUT commands OR Primary Key computing if attribute is tagged as key
     *
     * @param nextLink `key/putAttributeInput | (() => key/putAttributeInput)`
     */
    link(nextLink) {
        return ifThenElse(this.props.key, new AnyOfSchema_(this.elements, overwrite(this.props, { keyLink: nextLink })), new AnyOfSchema_(this.elements, overwrite(this.props, { putLink: nextLink })));
    }
    /**
     * Provide a custom validator for attribute in Primary Key computing
     *
     * @param nextKeyValidator `(keyAttributeInput) => boolean | string`
     */
    keyValidate(nextKeyValidator) {
        return new AnyOfSchema_(this.elements, overwrite(this.props, { keyValidator: nextKeyValidator }));
    }
    /**
     * Provide a custom validator for attribute in PUT commands
     *
     * @param nextPutValidator `(putAttributeInput) => boolean | string`
     */
    putValidate(nextPutValidator) {
        return new AnyOfSchema_(this.elements, overwrite(this.props, { putValidator: nextPutValidator }));
    }
    /**
     * Provide a custom validator for attribute in UPDATE commands
     *
     * @param nextUpdateValidator `(updateAttributeInput) => boolean | string`
     */
    updateValidate(nextUpdateValidator) {
        return new AnyOfSchema_(this.elements, overwrite(this.props, { updateValidator: nextUpdateValidator }));
    }
    /**
     * Provide a custom validator for attribute in PUT commands OR Primary Key computing if attribute is tagged as key
     *
     * @param nextValidator `(key/putAttributeInput) => boolean | string`
     */
    validate(nextValidator) {
        return ifThenElse(this.props.key, new AnyOfSchema_(this.elements, overwrite(this.props, { keyValidator: nextValidator })), new AnyOfSchema_(this.elements, overwrite(this.props, { putValidator: nextValidator })));
    }
    clone(nextProps = {}) {
        return new AnyOfSchema_(this.elements, overwrite(this.props, nextProps));
    }
    build(Action) {
        return new Action(this);
    }
}
