import { DynamoDBToolboxError } from '../../errors/index.js';
import { checkSchemaProps } from '../utils/checkSchemaProps.js';
export class ItemSchema {
    constructor(attributes) {
        this.type = 'item';
        this.attributes = attributes;
        this.props = {};
        this.savedAttributeNames = new Set();
        this.keyAttributeNames = new Set();
        this.requiredAttributeNames = {
            always: new Set(),
            atLeastOnce: new Set(),
            never: new Set()
        };
        for (const [attributeName, attribute] of Object.entries(attributes)) {
            const { key = false, required = 'atLeastOnce', savedAs = attributeName } = attribute.props;
            this.savedAttributeNames.add(savedAs);
            if (key) {
                this.keyAttributeNames.add(attributeName);
            }
            this.requiredAttributeNames[required].add(attributeName);
        }
    }
    get checked() {
        return Object.isFrozen(this.props);
    }
    check(path) {
        if (this.checked) {
            return;
        }
        checkSchemaProps(this.props, path);
        const attributesSavedAs = new Set();
        const keyAttributeNames = new Set();
        const requiredAttributeNames = {
            always: new Set(),
            atLeastOnce: new Set(),
            never: new Set()
        };
        for (const [attributeName, attribute] of Object.entries(this.attributes)) {
            const { savedAs: attributeSavedAs = attributeName, key: attributeKey, required: attributeRequired = 'atLeastOnce' } = attribute.props;
            if (attributesSavedAs.has(attributeSavedAs)) {
                throw new DynamoDBToolboxError('schema.item.duplicateSavedAs', {
                    message: `Invalid item attributes${path !== undefined ? ` at path '${path}'` : ''}: More than two attributes are saved as '${attributeSavedAs}'.`,
                    path,
                    payload: { savedAs: attributeSavedAs }
                });
            }
            attributesSavedAs.add(attributeSavedAs);
            if (attributeKey !== undefined && attributeKey) {
                keyAttributeNames.add(attributeName);
            }
            requiredAttributeNames[attributeRequired].add(attributeName);
        }
        for (const [attributeName, attribute] of Object.entries(this.attributes)) {
            attribute.check([path, attributeName].filter(Boolean).join('.'));
        }
        Object.freeze(this.props);
        Object.freeze(this.attributes);
        Object.freeze(this.savedAttributeNames);
        Object.freeze(this.keyAttributeNames);
        Object.freeze(this.requiredAttributeNames);
    }
}
