import { ifThenElse } from '../../utils/ifThenElse.js';
import { overwrite } from '../../utils/overwrite.js';
import { writable } from '../../utils/writable.js';
import { NullSchema } from './schema.js';
/**
 * Define a new schema of null type
 *
 * @param props _(optional)_ Schema Props
 */
export const nul = (props = {}) => new NullSchema_(props);
/**
 * Null attribute (warm)
 */
export class NullSchema_ extends NullSchema {
    /**
     * Tag attribute as required. Possible values are:
     * - `'atLeastOnce'` _(default)_: Required in PUTs, optional in UPDATEs
     * - `'never'`: Optional in PUTs and UPDATEs
     * - `'always'`: Required in PUTs and UPDATEs
     *
     * @param nextRequired SchemaRequiredProp
     */
    required(nextRequired = 'atLeastOnce') {
        return new NullSchema_(overwrite(this.props, { required: nextRequired }));
    }
    /**
     * Shorthand for `required('never')`
     */
    optional() {
        return this.required('never');
    }
    /**
     * Hide attribute after fetch commands and formatting
     */
    hidden(nextHidden = true) {
        return new NullSchema_(overwrite(this.props, { hidden: nextHidden }));
    }
    /**
     * Tag attribute as a primary key attribute or linked to a primary attribute
     */
    key(nextKey = true) {
        return new NullSchema_(overwrite(this.props, { key: nextKey, required: 'always' }));
    }
    /**
     * Rename attribute before save commands
     */
    savedAs(nextSavedAs) {
        return new NullSchema_(overwrite(this.props, { savedAs: nextSavedAs }));
    }
    /**
     * Provide a finite list of possible values for attribute
     * (For typing reasons, enums are only available as attribute methods, not as input props)
     *
     * @param enum Possible values
     * @example
     * string().enum('foo', 'bar')
     */
    enum(...nextEnum) {
        return new NullSchema_(overwrite(this.props, { enum: writable(nextEnum) }));
    }
    /**
     * Shorthand for `enum(constantValue).default(constantValue)`
     *
     * @param constantValue Constant value
     * @example
     * string().const('foo')
     */
    const(constant) {
        return ifThenElse(this.props.key, new NullSchema_(overwrite(this.props, { enum: [constant], keyDefault: constant })), new NullSchema_(overwrite(this.props, { enum: [constant], putDefault: constant })));
    }
    /**
     * Transform the attribute value in PUT commands OR Primary Key computing if attribute is tagged as key
     */
    transform(transform) {
        return new NullSchema_(overwrite(this.props, { transform }));
    }
    /**
     * Provide a default value for attribute in Primary Key computing
     *
     * @param nextKeyDefault `keyAttributeInput | (() => keyAttributeInput)`
     */
    keyDefault(nextKeyDefault) {
        return new NullSchema_(overwrite(this.props, { keyDefault: nextKeyDefault }));
    }
    /**
     * Provide a default value for attribute in PUT commands
     *
     * @param nextPutDefault `putAttributeInput | (() => putAttributeInput)`
     */
    putDefault(nextPutDefault) {
        return new NullSchema_(overwrite(this.props, { putDefault: nextPutDefault }));
    }
    /**
     * Provide a default value for attribute in UPDATE commands
     *
     * @param nextUpdateDefault `updateAttributeInput | (() => updateAttributeInput)`
     */
    updateDefault(nextUpdateDefault) {
        return new NullSchema_(overwrite(this.props, { updateDefault: nextUpdateDefault }));
    }
    /**
     * Provide a default value for attribute in PUT commands OR Primary Key computing if attribute is tagged as key
     *
     * @param nextDefault `key/putAttributeInput | (() => key/putAttributeInput)`
     */
    default(nextDefault) {
        return ifThenElse(this.props.key, new NullSchema_(overwrite(this.props, { keyDefault: nextDefault })), new NullSchema_(overwrite(this.props, { putDefault: nextDefault })));
    }
    /**
     * Provide a **linked** default value for attribute in Primary Key computing
     *
     * @param nextKeyLink `keyAttributeInput | ((keyInput) => keyAttributeInput)`
     */
    keyLink(nextKeyLink) {
        return new NullSchema_(overwrite(this.props, { keyLink: nextKeyLink }));
    }
    /**
     * Provide a **linked** default value for attribute in PUT commands
     *
     * @param nextPutLink `putAttributeInput | ((putItemInput) => putAttributeInput)`
     */
    putLink(nextPutLink) {
        return new NullSchema_(overwrite(this.props, { putLink: nextPutLink }));
    }
    /**
     * Provide a **linked** default value for attribute in UPDATE commands
     *
     * @param nextUpdateLink `unknown | ((updateItemInput) => updateAttributeInput)`
     */
    updateLink(nextUpdateLink) {
        return new NullSchema_(overwrite(this.props, { updateLink: nextUpdateLink }));
    }
    /**
     * Provide a **linked** default value for attribute in PUT commands OR Primary Key computing if attribute is tagged as key
     *
     * @param nextLink `key/putAttributeInput | (() => key/putAttributeInput)`
     */
    link(nextLink) {
        return ifThenElse(this.props.key, new NullSchema_(overwrite(this.props, { keyLink: nextLink })), new NullSchema_(overwrite(this.props, { putLink: nextLink })));
    }
    /**
     * Provide a custom validator for attribute in Primary Key computing
     *
     * @param nextKeyValidator `(keyAttributeInput) => null | string`
     */
    keyValidate(nextKeyValidator) {
        return new NullSchema_(overwrite(this.props, { keyValidator: nextKeyValidator }));
    }
    /**
     * Provide a custom validator for attribute in PUT commands
     *
     * @param nextPutValidator `(putAttributeInput) => null | string`
     */
    putValidate(nextPutValidator) {
        return new NullSchema_(overwrite(this.props, { putValidator: nextPutValidator }));
    }
    /**
     * Provide a custom validator for attribute in UPDATE commands
     *
     * @param nextUpdateValidator `(updateAttributeInput) => null | string`
     */
    updateValidate(nextUpdateValidator) {
        return new NullSchema_(overwrite(this.props, { updateValidator: nextUpdateValidator }));
    }
    /**
     * Provide a custom validator for attribute in PUT commands OR Primary Key computing if attribute is tagged as key
     *
     * @param nextValidator `(key/putAttributeInput) => null | string`
     */
    validate(nextValidator) {
        return ifThenElse(this.props.key, new NullSchema_(overwrite(this.props, { keyValidator: nextValidator })), new NullSchema_(overwrite(this.props, { putValidator: nextValidator })));
    }
    clone(nextProps = {}) {
        return new NullSchema_(overwrite(this.props, nextProps));
    }
    build(Action) {
        return new Action(this);
    }
}
